/*****************************************************************************
 *
 * Written 2008 by Matthias Kirchner, matze@renhcrik.de
 * using some routines from W. Reissnegger and B. Walle
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with puliwera;; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ******************************************************************************/

#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "display.h"
#include "puliwera.h"
#include "info.h"

//static unsigned int scroll_next;	// time for the next scroll event
//static int scroll_delta	= 1;		// 1 or -1, will be added to idx
//static int scroll_idx	= 0;		// scroll index

#define lcd_printf(_arg...)         \
    {   \
        char buf[512];                                  \
        sprintf(buf, _arg);                             \
        socket_send(lcd_fd(display), buf);      \
    }
#define xy_printf(_col,_line,_arg...)           \
    {   \
        char _buf[512];    \
        char _cmd[512];    \
        sprintf(_buf,_arg);         \
        sprintf(_cmd,"widget_set PULIWERA line%d %d %d {%s}\n",_line,_col,(_line)+1,_buf); \
        socket_send(lcd_fd(display), _cmd);    \
    }
/*
aus xy_printf:
        printf("\033[%d;%dH\033[0;42;30m%-40.40s\033[0m",_line+1+18,_col+1+20,_buf); \
	fflush(stdout); \
*/

static int lcd_open(display_t * display);
static void lcd_close(display_t * display);
static int lcd_fd(display_t * display);
static void lcd_handle_event(display_t * display);
static void lcd_show(display_t * display, void *ctx, unsigned int type);
static void lcd_show_menu(display_t * display, void *ctx);
static void lcd_cmd(display_t * display, unsigned int cmd, void *arg);

static struct session_d
{
	screen_t *screen_stack[512];
	int screen_stack_idx;
}
session[1];
static struct session_d *sp = session;


// by W. Reissnegger
struct lcd_display_d
{
	display_t parent;

	int sock;		// socket fd
	int lines;		// number of display lines
	int columns;		// number of display columns

#define m_sock      ((struct lcd_display_d *) display)->sock
#define m_lines     ((struct lcd_display_d *) display)->lines
#define m_columns   ((struct lcd_display_d *) display)->columns
};

// by W. Reissnegger
struct lcd_display_d lcd_display = {
	{
	      open:			lcd_open,
	      close:		lcd_close,
	      fd:			lcd_fd,
	      ev_handler:	lcd_handle_event,
	      show:			lcd_show,
	      cmd:			lcd_cmd,
	 },
	-1,			// sock
	2,			// lines
	40			// columns
	};

display_t *display_list[] = {(void *) &lcd_display,NULL};

// by W. Reissnegger, but modified
int init_display()
{
	int rc = 0;
	display_t *display=0;
	display = display_list[0];
	rc |= display->open(display);
	if (debug) printf("rc: %i\n",rc);
	return rc;

}

// by W. Reissnegger, but modified
static int lcd_open(display_t * display)
{
	const char *server = DEFAULT_SERVER;

	int port = DEFAULT_PORT;
	int i;
//	char *s;

	// set default LCD size
	m_columns = 40;
	m_lines = 2;

	// wait for the server to start up
	usleep(500000);

	m_sock = socket_connect(server, port);
	if (debug) printf("Socket connected\n");
	if (m_sock <= 0)
	{
		printf("Error connecting to LCD server %s on port %i.\n"
		       "Check to see that the server is running and operating normally.\n", server,
		       port);
		return -1;
	}
	socket_send(m_sock, "hello\n");
	usleep(500000);		// wait for the server to say hi.
	if (debug) printf("sent \"hello\"\n");
	// Init the screen
	lcd_printf("screen_add PULIWERA\n");
	lcd_printf("screen_set PULIWERA -backlight on -priority 16 -name PULIWERA -heartbeat on\n");
	if (debug) printf("screen added\n");
	// Add key requests for the LCD keys.
	for (i = 0; i < m_lines; ++i)
	{
		lcd_printf("widget_add PULIWERA line%d string\n", i);
	}

	lcd_printf("client_add_key 1\n");
	lcd_printf("client_add_key 2\n");
	lcd_printf("client_add_key 3\n");
	lcd_printf("client_add_key 4\n");
/*	lcd_printf("client_add_key Up\n");
	lcd_printf("client_add_key Down\n");
	lcd_printf("client_add_key Enter\n");*/
	if (debug) printf("keys added\n");

	return 0;
}

screen_t *display_current_screen()
{
//	return sp->screen_stack[0];
	return sp->screen_stack[sp->screen_stack_idx - 1];
}

// by W. Reissnegger
void display_push(screen_t * screen)
{
	assert(sp->screen_stack_idx < 64);
	sp->screen_stack[sp->screen_stack_idx++] = screen;
}

// by W. Reissnegger
void display_pop()
{
	// we never pop the first screen, because it is the main menu screen.
	//
	if (sp->screen_stack_idx > 1)
	{
		screen_t *current = sp->screen_stack[--sp->screen_stack_idx];

		if (current->delete)
		{
			(current->delete) (current);
		}
	}
}

// by W. Reissnegger, but modified
void display_show()
{
	screen_t *screen = display_current_screen();
	int idx;
	display_t *display;

	assert(NULL != screen);

	if (screen->update)
	{
		screen->update(screen);
	}

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		if (display->show)
		{
			display->show(display, screen->ctx, screen->type);
		}
	}
}

// by W. Reissnegger, but modified
void display_close()
{
	display_t *display;
	display = display_list[0];
	if (display->close)
	{
		display->close(display);
	}
if (debug) printf("display closed\n");
}

// by W. Reissnegger
static void lcd_close(display_t * display)
{
	socket_close(m_sock);
}

// by W. Reissnegger
static int lcd_fd(display_t * display)
{
	return m_sock;
}

static unsigned int lcd_translate_key(const char key)
{
	switch (key)
	{
		case '1':	return KEY_UP;
		case '2':	return KEY_DOWN;
		case '3':	return KEY_SELECT;
		case '4':	return KEY_ESC;
	}
	return 0;
}

// by W. Reissnegger
void display_handle_key(unsigned int key)
{
	screen_t *screen = display_current_screen();

	assert(NULL != screen);

	if (screen->handle_key)
	{
		screen->handle_key(screen, key);
	}
}

void handle_key(unsigned int key)
{
	if (debug)
	{
		printf("KEY_ESC=%i\n",KEY_ESC);
		printf("keyvalue: %i\n",key);
	}
//	if (key == KEY_ESC)	display_pop();
	display_handle_key(key);
}

// The token handling code below has been snipped out of the LCDproc client.
static void lcd_handle_event(display_t * display)
{
	int i;

	char buf[8192];
	char *argv[256];
	int argc;
	int newtoken;
	int len;

	len = socket_receive(m_sock, buf, sizeof(buf));

	// Handle server input...
	while (len > 0)
	{
		// Now split the string into tokens...
		//
		argc = 0;
		newtoken = 1;
		for (i = 0; i < len; i++)
		{
			switch (buf[i])
			{
			case ' ':
				newtoken = 1;
				buf[i] = 0;
				break;
			default:	// regular chars, keep tokenizing
				if (newtoken)
				{
					argv[argc] = buf + i;
					argc++;
				}
				newtoken = 0;
				break;
			case '\0':
			case '\n':
				buf[i] = 0;
				if (argc > 0)
				{
					if (0 == strcmp(argv[0], "listen"))
					{
						printf("Listen %s\n", argv[1]);
					}
					else if (0 == strcmp(argv[0], "ignore"))
					{
						printf("Ignore %s\n", argv[1]);
					}
					else if (0 == strcmp(argv[0], "key"))
					{
						if (debug) printf("Key %s\n", argv[1]);
						handle_key(
							lcd_translate_key(*(argv[1])));
					}
					else if (0 == strcmp(argv[0], "menu"))
					{
						printf("Menu %s\n", argv[1]);
					}
					else if (0 == strcmp(argv[0], "connect"))
					{
						int a;

						for (a = 1; a < argc; a++)
						{
							if (0 == strcmp(argv[a], "wid"))
							{
								a++;
								m_columns = atoi(argv[a]);
							}
							else if (0 == strcmp(argv[a], "hgt"))
							{
								a++;
								m_lines = atoi(argv[a]);
							}
							else if (0 == strcmp(argv[a], "cellwid"))
							{
								a++;
							}
							else if (0 == strcmp(argv[a], "cellhgt"))
							{
								a++;
							}
						}
						lcd_printf("client_set -name PuLiWeRa\n");
					}
					else if (0 == strcmp(argv[0], "bye"))
					{
						if (debug) printf("bye\n");
					}
					else if (0 == strcmp(argv[0], "success"))
					{
						// ignore  //if (debug) printf("success\n");
					}
					else
					{
						// print any remaining
						// unhandled messages
						int j;

						for (j = 0; j < argc; j++)
							printf("%s ", argv[j]);
						printf("\n");
					}
				}

				// Restart tokenizing
				argc = 0;
				newtoken = 1;
				break;
			}	// switch(buf[i])
		}

		len = socket_receive(m_sock, buf, sizeof(buf));
	}
}

// by W. Reissnegger, but modified
static void lcd_show_menu(display_t * display, void *ctx)
{
	int i;
	int line;

	// Check, if the selection is still within our display area
	if (c_display_top_line + m_lines <= c_selection)
	{	
		c_display_top_line = c_selection - (m_lines - 1); // adjust downwards
	}
	else if (c_display_top_line > c_selection)
	{
		c_display_top_line = c_selection; // adjust upwards
	}

	// Loop through the visible lines and print them
	for (i = 0, line = c_display_top_line; i < m_lines; i++, line++)
	{
		file_entry_t *entry = list_data(list_nth(c_start, line));

		if (entry)
		{
			char *name;

			name = c_view_fullpath ? entry->fullpath : entry->name;

			xy_printf(1, i, "%c %s",
				c_selection == i + c_display_top_line ?
				entry->marked ? '#' : '>' :
				entry->marked ? '*' : ' ',
//				FILE_TYPE_DIR == file_type(entry) ? "+" : "",
				&name[c_left_margin]);
		}
		else
		{
			xy_printf(1, i, "-");
		}
	}
}

// by W. Reissnegger
static void lcd_show_info(display_t * display, void *ctx)
{
	int line;

	for (line = 0; line < m_lines; ++line)
	{
		xy_printf(1, line, c_text[line]);
	}
}

// by W. Reissnegger, but modified
static void lcd_show(display_t * display, void *ctx, unsigned int type)
{
	assert(NULL != ctx);
	assert(NULL != display);

//		lcd_show_menu(display, ctx);

	switch (type)
	{
	case TYPE_MENU:
//	case TYPE_SONGLIST_MENU:
		lcd_show_menu(display, ctx);
		break;
//	case TYPE_SEARCH:
//		lcd_display_show_search(display, ctx);
//		break;*/
 	case TYPE_INFO:
 		lcd_show_info(display, ctx);
 		break;
// 	case TYPE_SONG:
// 		lcd_display_show_song(display, ctx);
// 		break;

//	default:
		// ignore
	;
	}
}

// by W. Reissnegger, not used
static void lcd_cmd(display_t * display, unsigned int cmd, void *arg)
{
	if (debug) printf("lcd_cmd benutzt");

/*	scroll_idx	= 0;
	scroll_delta	= 1;
	
	// Set the initial time it takes before the song name starts scrolling.
	//
//	scroll_next	= get_timestamp() + SONG_SCROLL_INITIAL_WAIT; */
}

// by W. Reissnegger
void display_handle_event(fd_set * fds)
{
	int idx;
	display_t *display;
	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		if (!display->fd)
			continue;
		if (FD_ISSET(display->fd(display), fds) && display->ev_handler)
		{
			display->ev_handler(display);
		}
	}
}

// by W. Reissnegger
int display_fds(fd_set * fds)
{
	int idx;
	int highest_fd = 0;
	display_t *display;

	for (idx = 0; NULL != (display = display_list[idx]); ++idx)
	{
		int fd;
		if (!display->fd)
			continue;
		fd = display->fd(display);
		FD_SET(fd, fds);
		if (fd > highest_fd)
			highest_fd = fd;
	}
	return highest_fd;
}
