/****************************************************************************
 *
 *  Written 2008 by Matthias Kirchner, matze@renhcrik.de
 *  using some routines from W. Reissnegger and B. Walle
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#ifndef DISPLAY_H_
#define DISPLAY_H_

#include <sys/select.h>

#include "puliwera.h"
#include "socket.h"
#include "list.h"

enum
{
	KEY_NONE = 333,
	KEY_UP,
	KEY_DOWN,
	KEY_SELECT,
	KEY_ESC,
};

extern struct lcd_display_d lcd_display;

// by W. Reissnegger
typedef struct screen_d
{
	unsigned int type;	// menu, status type checking magic number

	void (*update) (struct screen_d *);
	void (*delete) (struct screen_d *);
	int (*handle_key) (struct screen_d *, int);

	// This is a pointer to the context object of the 'class' which derives
	// from the screen 'class'. Th _new() function of the deriving 'class'
	// MUST initialize this pointer to its context object, otherwise
	// display_show() will crash.
	//
	void *ctx;
}
screen_t;

// by W. Reissnegger
typedef struct display_d
{
	int (*open) (struct display_d *);	// Display open function (0 == success)
	void (*close) (struct display_d *);	// Display close function
	int (*fd) (struct display_d *);	// Display's fd (-1 if not available)
	void (*ev_handler) (struct display_d *);	// Display event handler (if needed)
	void (*show) (struct display_d *, void *, unsigned int);	// Display show function
	void (*cmd) (struct display_d *, unsigned int, void *);	// Display command handler
#define CMD_CLEAR       1
}
display_t;

// by W. Reissnegger
typedef struct file_entry_d
{
	char name[512];
	char fullpath[512];
	void (*handler) (void *);
#define FILE_ENTRY_FILE         0
#define FILE_ENTRY_MENU         1
	int type;
	int marked;
}
file_entry_t;

// by W. Reissnegger
typedef struct menu_display_context_d
{
	list_t *start;		// start of the menu list
	int selection;		// currently selected line
	int view_fullpath;	// normal, fullpath view flag
	int display_top_line;	// entry display at first line
	int left_margin;	// Left margin of menu display

// These are accessor macros. Make sure that ctx is of type
// menu_display_context_t* when using these...
#define c_start                 ((menu_display_context_t *) ctx)->start
#define c_selection             ((menu_display_context_t *) ctx)->selection
#define c_view_fullpath         ((menu_display_context_t *) ctx)->view_fullpath
#define c_display_top_line      ((menu_display_context_t *) ctx)->display_top_line
#define c_left_margin           ((menu_display_context_t *) ctx)->left_margin
}
menu_display_context_t;

/******************************************************************************/

void display_session_set(unsigned int idx);
unsigned int display_session_idx();

void display_push(screen_t * screen);
void display_pop();
screen_t *display_current_screen();

int init_display();

void display_open();
void display_close();
void display_show();
void display_cmd(unsigned int cmd, void *arg);
int display_fds(fd_set * fds);
void display_handle_event(fd_set * fds);
void display_handle_key(unsigned int key);

#endif
