/****************************************************************************
 *
 *   Copyright (C) 2003, 2004 Wolfgang G. Reissnegger, All Rights Reserved
 *
 *   gnagflow@users.sourceforge.net                                    
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <stdio.h>

#include "display.h"
#include "info.h"

/******************************************************************************/

typedef struct info_d
{
	// Screen base 'class' object
	//
	screen_t parent;

	// display context object
	//
	info_display_context_t ctx;

	int timeout;
	unsigned int screen_init;

#define m_ctx           ((info_t *) screen)->ctx
#define m_text          ((info_t *) screen)->ctx.text
#define m_timeout       ((info_t *) screen)->timeout
#define m_screen_init   ((info_t *) screen)->screen_init
}
info_t;

unsigned int get_timestamp()
{
	struct timeval now;

	gettimeofday(&now, NULL);
	return now.tv_sec * 1000 + now.tv_usec / 1000;
}

/******************************************************************************/

static void info_update(screen_t * screen)
{
	assert((NULL != screen) && (TYPE_INFO == screen->type));

	if (0 != m_timeout)
	{
		unsigned int ms_now;

		// Get the current time to calculate if we have to 'pop'
		// ourselves from the display stack.
		//
		ms_now = get_timestamp();

		if ((ms_now - m_screen_init) > m_timeout)
		{
			display_pop();
		}
	}
}


/******************************************************************************/

static int info_handle_key(screen_t * screen, int key)
{
	// If we receive a key, we'll pop ourselves from the
	// display stack.
	//
	if (debug) printf("info_handle_key\n");
	display_pop();
	return KEY_HANDLED;
}


/******************************************************************************/

static void info_delete(screen_t * screen)
{
	if (debug) printf("info_delete\n");
	assert((NULL != screen) && (TYPE_INFO == screen->type));
	free(screen);
}


/******************************************************************************
 *
 * info_set_text
 *
 * Updates the info screen with the given text
 *
 ******************************************************************************/

void info_set_text(screen_t * screen, int line, const char *text)
{
	assert((NULL != screen) && (TYPE_INFO == screen->type));
	assert(NULL != text);
	assert(line < INFO_MAX_LINES);

	save_strcpy(m_text[line], text);
}


/******************************************************************************
 *
 * info_new
 *
 * Initializes a information screen and sets the given text and timeout.
 *
 * timeout defines how long the screen should appear (in milliseconds) before
 * it automatically removes itself. timeout == 0 means forever --- the caller
 * will have to pop the screen manually.
 *
 ******************************************************************************/

screen_t *info_new(const char *text, int timeout)
{
	screen_t *screen;

	assert(NULL != text);

	// NOTE: We actually allocate a info_t object, but we handle it with
	// its parent 'class' (screen_t) pointer.
	//
	screen = (screen_t *) malloc(sizeof(info_t));
	assert(screen);

	bzero(screen, sizeof(info_t));
	screen->type		= TYPE_INFO;
	screen->update		= info_update;
	screen->delete		= info_delete;
	screen->handle_key	= info_handle_key;
	screen->ctx		= &m_ctx;

	info_set_text(screen, 0, text);
	m_screen_init = get_timestamp();
	m_timeout = timeout;

	return screen;
}
