/****************************************************************************
 *
 *   originally from Wolfgang G. Reissnegger, 
 *   shortened by Matthias Kirchner
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the license, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program;  if not, write to the
 *       Free Software Foundation, Inc.,
 *       59 Temple Place - Suite 330,
 *       Boston, MA 02111-1307
 *
 ****************************************************************************/

#include <stdlib.h>
#include <assert.h>

#include "list.h"


/******************************************************************************/

struct list_d
{
	struct list_d *next;	// next pointer
	struct list_d *prev;	// previous element pointer
	void *data;		// list data
};


/*********************************************************************************
 *
 * Creates a new list node, sets the data pointer and appends it to the given list.
 *
 *********************************************************************************/

list_t *list_add(list_t *node, void *data)
{
	list_t *new_elem = (list_t *) malloc(sizeof(list_t));

	assert(NULL != new_elem);

	new_elem->next = NULL;
	new_elem->data = data;

	if (node)
	{
		// Elements exist in list
		//
		list_t *last = list_last(node);

		last->next = new_elem;
		new_elem->prev = last;

		return node;
	}
	else
	{
		// List is empty, this is the first element
		//
		new_elem->prev = NULL;

		return new_elem;
	}
}

list_t *list_next(list_t *node)
{
	return node ? node->next : node;
}

/******************************************************************************
 *
 * Returns the data pointer of a given list node.
 *
 ******************************************************************************/

void *list_data(list_t *node)
{
	return node ? node->data : node;
}

/******************************************************************************
 *
 * Returns the first element of a list.
 *
 ******************************************************************************/

list_t *list_first(list_t *node)
{
	if (node)
	{
		while (node->prev)
			node = node->prev;
	}

	return node;
}


/******************************************************************************
 *
 * Returns the last element of a list.
 *
 ******************************************************************************/

list_t *list_last(list_t *node)
{
	if (node)
	{
		while (node->next)
			node = node->next;
	}
	return node;
}

/******************************************************************************
 *
 * This function only frees the list nodes. It does NOT DELETE ANY DATA
 * contained in the list.
 *
 ******************************************************************************/

void list_free(list_t *node)
{
	list_t *current = list_first(node);

	while (current)
	{
		list_t *next = list_next(current);

		free(current);
		current = next;
	}
}

/******************************************************************************
 *
 * Returns the n-th element of the list.
 *
 ******************************************************************************/

list_t *list_nth(list_t *list, int n)
{
	while (n-- && list)
		list = list_next(list);

	return list;
}

/******************************************************************************
 *
 * Returns the number of list elements in the list.
 *
 ******************************************************************************/

int list_len(list_t *list)
{
	int len = 0;

	while (list)
	{
		list = list_next(list);
		len++;
	}
	return len;
}
